'******************************************************************************
'*  pm85.spin - a recompiling Intel 8085 emulator
'*
'*  (c) Juergen Buchmueller <pullmoll@t-online.de>
'*
'* $Id: pm85.spin,v 1.5 2010-03-11 11:19:30 pm Exp $
'******************************************************************************
CON
	_xinfreq		= 5_000_000		' Quartz is 5MHz
	_clkmode		= xtal1 + pll16x	' System clock is 80MHz

	c_flag			=	%00000001	' carry
	v_flag			=	%00000010	' overflow
	p_flag			=	%00000100	' parity (also overflow)
	x3_flag			=	%00001000	' undocumented x3
	h_flag			=	%00010000	' auxiliaray (half) carry
	x5_flag			=	%00100000	' undocumented x5
	z_flag			=	%01000000	' zero
	s_flag			=	%10000000	' sign

	IM_SID			=	%10000000	' RIM/SIM masks
	IM_I75			=	%01000000
	IM_I65			=	%00100000
	IM_I55			=	%00010000
	IM_IE			=	%00001000
	IM_M75			=	%00000100
	IM_M65			=	%00000010
	IM_M55			=	%00000001

	ADDR_TRAP		=	$24		' vector for trap (non maskable)
	ADDR_RST55		=	$2c		' vector for RST5.5
	ADDR_RST65		=	$34		' vector for RST6.5
	ADDR_RST75		=	$3c		' vector for RST7.5
	ADDR_INTR		=	$38		' vector for classic interrupt
	ADDR_RSTV		=	$40		' vector for 8085 RST V

	PATCH_SHIFT		=	23

	tvPin			=	15
	CONOUT_PORT		=	$c0
'
' Direct screen access region for debug output
'
	SCR_Y			=	6
	SCR_X			=	0
	SCR_W			=	40
	SCR_H			=	18
'
' Define this to watch the compiler in action
#define DEBUG
'
' Define this to have the 8085 opcodes instead of the plain 8080 ones
' #define CPU_8085
'
' Define this to always only compile one instruction in the cog cache
' #define SSTEP

'******************************************************************************
'
' Structure of a compiled 8085 opcode.
'
'  long #     meaning
' ---------------------------------------------------------------------
'       0     contains the i8085 PC for the opcode (of the first byte)
'       1     code pasted from the code snippet
'      ...
'       n+1    last code long
'       n+2   contains the i8085 PC of the next opcode
'
'******************************************************************************

VAR
	long  cog

OBJ
	tv	: "Debug_1pinTV"

PUB start(ram, io) : okay

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
'' ram is a pointer to i8085 RAM space
'' io is a pointer to i8085 I/O space

	stop
        tv.start(tvPin)        'start the Debug 1pinTV driver
        tv.chr(0)              'clear screen
        tv.str(string("PM85 DRC firing up...",13))

	reloc := @entry - $10
	ram_base := ram
	io_base := io
	conout := tv.RENDEZVOUS
	scr := tv.SCRPTR

	okay := cog := COGNEW(@entry, 0) + 1

PUB stop

'' Stop CPU simulator - frees a cog

	if cog
		cogstop(cog~ - 1)

DAT
'---------------------------------------------------------------------------------------------------------

		org	0

'---------------------------------------------------------------------------------------------------------
entry
hubptr					' contains hub RAM address for ea (overwrites jmp)
		jmp	#startup

'******************************************************************************
'
' cog variables
'
reloc		long	0-0		' relocation address for tables + functions
ram_base	long	0-0		' base address of the i8085 RAM
io_base		long	0-0
conout		long	0-0		' console output (pRENDEZVOUS of Debug_1pinTV)
scr		long	0-0		' pointer to the screen buffer
#ifdef	DEBUG
scrx		long	SCR_X
scry		long	SCR_Y
#endif

ea		long	0		' effective address
opcode		long	0		' opcode
halt		long	0		' HALT flag, set by opcode 76

alu		long	0		' arithmetic logic unit (sort of :)
tmp		long	0		' generic temporary
aux		long	0		' auxiliary flag temporary

dst_pc		long	0		' compiler output program counter
max_pc		long	0		' max. address for compiler output
snippet1	long	0		' start address of current code snippet in cog RAM
adj_06					' re-use this long in DAA
count		long	0		' count of longs to copy
adj_60					' re-use this long in DAA
patch		long	0		' opcode patch coming from the snippets_xx_table

t1		long	0
t2		long	0
t3		long	0
low_16bits	long	$0000ffff

snippets_table	long	@@@snippets

#ifdef	DEBUG
'time		long	80_000_000
time		long	80_000_000
char		long	0
#endif

C		long	0		' C register
B		long	0		' B register
E		long	0		' E register
D		long	0		' D register
L		long	0		' L register
H		long	0		' H register
F		long	z_flag		' F register (flags)
A		long	0		' A register
SP		long	0		' stack pointer
PC		long	0		' program counter
PC0		long	0		' program counter at the start of this #compile
PC1		long	0		' program counter at the start of this instruction
R		long	0		' R register (refresh address)
R2_I		long	0		' bit #7 of last write to R register, I register in MSB

startup
'---------------------------------------------------------------------------------------------------------
' i8085 compiler
'---------------------------------------------------------------------------------------------------------
compile
		mov	dst_pc, #ORIGIN			' start over to compile to ORIGIN++
		mov	max_pc, #$1f0
		mov	PC0, PC				' remember the program counter of this compiled block
compile_next
#ifdef	SSTEP
		cmp	dst_pc, #ORIGIN WZ
	if_nz	jmp	#execute
#endif
		mov	PC1, PC				' save PC at start of this opcode
		call	#rd_opcode			' get opcode

		mov	tmp, snippets_table
#ifdef	DEBUG
		mov	hexval, PC1
		call	#hex_w
		mov	char, #$3a
		call	#scr_chr
		mov	hexval, opcode
		call	#hex_b
		call	#scr_space
		mov	hexval, B
		call	#hex_b
		mov	hexval, C
		call	#hex_b
		call	#scr_space
		mov	hexval, D
		call	#hex_b
		mov	hexval, E
		call	#hex_b
		call	#scr_space
		mov	hexval, H
		call	#hex_b
		mov	hexval, L
		call	#hex_b
		call	#scr_space
		mov	hexval, A
		call	#hex_b
		mov	hexval, F
		call	#hex_b
		call	#scr_space
		mov	hexval, SP
		call	#hex_w
		call	#scr_space
#endif
compile_op
		shl	opcode, #2			' * 4
		add	opcode, tmp			' offset into one of the code snippets tables
		rdlong	ea, opcode			' get the address + patch of the code snippet
		mov	patch, ea			' save a copy in patch for later
		add	ea, reloc
		rdlong	count, ea			' get number of longs to copy

#ifdef	DEBUG
		mov	hexval, dst_pc
		sub	hexval, #ORIGIN
		call	#hex_w
		call	#scr_cr
#endif

		' verify that this code snippet still fits in cog RAM
		mov	tmp, dst_pc			' destination
		add	tmp, count			' + count longs
		add	tmp, #3				' + 3 longs (need to fit final PC/jump #compile)
		cmp	tmp, max_pc WZ, WC		' < last possible cog RAM address?
	if_ae	mov	PC, PC1				' compile this instruction again
	if_ae	jmp	#execute			' no, now execute what we compiled

		movd	:padr0, dst_pc
		nop
:padr0		mov	0-0, PC1			' store PC of the instruction in the snippet's first long
							' the upper 6 bits are zero, so this is a PASM NOP
		add	dst_pc, #1
		mov	snippet1, dst_pc
		tjz	count, #compile_next		' count == 0: copy nothing at all
:copy1
		movd	:padr2, dst_pc			' store destination address in rdlong opcode
		add	ea, #4				' next source address (+ 1 long)
		rdlong	tmp, ea
		shr	tmp, #(32-6) WZ, NR		' see if this is a NOP (paste code snippet marker)
	if_z	call	#paste_snippet			' yes, paste the corresponding snippet of code
:padr2	if_nz	mov	0-0, tmp			' store the PASM opcode
	if_nz	add	dst_pc, #1			' next cog RAM address
		djnz	count, #:copy1			' copy the code snippet

		shr	patch, #PATCH_SHIFT WZ		' get patcher to bits 8..0
	if_nz	jmp	patch				' go to the patcher
		jmp	#compile_next

'*******************************************************
' Patch an imm16 into the code and then prepare
' the execution of the compiled code
'
execute_imm16
		call	#do_patch_imm16

'*******************************************************
' Prepare execution of the compiled code
'
execute
#ifdef	DEBUG
		mov	tmp, cnt
		add	tmp, time
		waitcnt	tmp, time
#endif
		' store the current i8085 PC in the code
		movd	:padr0, dst_pc			' store destination address in the mov opcode
		add	dst_pc, #1
:padr0		mov	0-0, PC				' next PC
		' store a "jmp #compile" at the end of the cog RAM at dst_pc
		movd	:padr1, dst_pc			' store destination address in the mov opcode
		mov	ea, dst_pc			' save dst_pc in ea
:padr1		mov	0-0, jmp_compile_op
		jmp	#ORIGIN				' now execute what we compiled

jmp_compile_op	jmp	#compile

'*******************************************************
' Paste another snippet of code into the current one
' snippet source address (in cog RAM) is in tmp
'
paste_snippet
		add	tmp, reloc
		rdlong	aux, tmp			' fetch the number of longs to copy
:copy		movd	:padr0, dst_pc			' store destination address in rdlong opcode
		add	tmp, #4				' next source address (+ 1 long)
:padr0		rdlong	0-0, tmp			' read the PASM opcode to cog RAM
		add	dst_pc, #1			' next cog RAM address
		sub	count, #1			' decrement count also
		djnz	aux, #:copy			' copy the code snippet
		cmp	tmp, tmp WZ			' make sure the Z flag is set
paste_snippet_ret
		ret

'*******************************************************
' Patch an immediate 16 bit (word) load in snippet 
'
do_patch_imm16
		sub	max_pc, #1			' one less long available
		movd	:padr2, snippet1		' set patch address for address in code
		movd	:padr1, max_pc			' set patch address for long at end of cog RAM
		call	#rd_opword
:padr1		mov	0-0, alu			' store word in long at the end of cog RAM
:padr2		movs	0-0, max_pc			' patch the code with the address of the word
do_patch_imm16_ret
		ret

'*******************************************************
' Patch an immediate 16 bit (word) load in snippet 
'
patch_imm16
		call	#do_patch_imm16
		jmp	#compile_next

'*******************************************************
' Try and see if the program counter stored in alu
' is found in the compiled code and if so, patch the
' 2nd instruction "mov ea, #0-0" to the address after
' the NOP containing the PC
'
patch_compiled_pc
		cmp	alu, PC0 WZ, WC
	if_b	jmp	#patch_compiled_pc_ret		' new PC is below this compiled block
		cmp	alu, PC WZ, WC
	if_a	jmp	#patch_compiled_pc_ret		' new PC is beyond the current address
		mov	ea, dst_pc			' get the end of the compiler output
		sub	ea, #1
:loop		movs	:padr0, ea			' and start to search backwards
		sub	ea, #1
:padr0		mov	aux, 0-0			' get the code word
		cmp	aux, alu WZ			' found the expected PC?
	if_z	jmp	#:found				' yes, :found
		cmp	ea, #ORIGIN WZ, WC		' hit bottom of compiled code?
	if_b	jmp	#patch_compiled_pc_ret		' not found
		jmp	#:loop
:found		add	tmp, #1				' + 1 into the snippet's code
		movd	:padr1, tmp			' set patch address
		add	ea, #2				' address after the tinted NOP
:padr1		movs	0-0, ea				' patch the code with the found snippet's address
patch_compiled_pc_ret
		ret

'*******************************************************
' Patch an absolute jump address in snippet
'
patch_jp_imm16
		movd	:padr1, snippet1		' address of the "mov aux 0-0"
		sub	max_pc, #1
		movd	:padr2, max_pc
		call	#rd_opword			' get the imm16 from (PC++)
:padr1		movs	0-0, max_pc			' store address of new PC word in the snippet
:padr2		mov	0-0, alu			' store alu in the top cog RAM slot
		mov	tmp, snippet1
		call	#patch_compiled_pc
		jmp	#compile_next

'*******************************************************
' Patch an immediate 8 bit (byte) in snippet
'
patch_imm8
		movd	:padr, snippet1			' set patch address
		call	#rd_opcode
:padr		movs	0-0, opcode			' patch the code with the byte
		jmp	#compile_next

'*******************************************************
' Patch a relative jump address in snippet
'
patch_jr_disp8
		movd	:padr1, snippet1
		sub	max_pc, #1
		movd	:padr2, max_pc
		call	#rd_opcode
		mov	alu, opcode
		shl	alu, #24			' sign extend the byte
		sar	alu, #24			' into a long
		add	alu, PC				' alu = new target address
:padr1		movs	0-0, max_pc			' store address of new PC word in the snippet
:padr2		mov	0-0, alu			' store alu in the top cog RAM slot
		mov	tmp, snippet1
		call	#patch_compiled_pc
		jmp	#compile_next

'*******************************************************
' Patch the address of the snippet into the snippet
'
patch_self_1
		movd	:padr, snippet1
		nop
:padr		movs	0-0, snippet1
		jmp	#compile_next

'*******************************************************
' A snippet has modified compiled code. We need to find
' the i8085 PC of the following snippet (or the end of
' of the compiler sweep) and recompile from there
snippet_pc
		and	tmp, #$1ff			' mask the return address of the modifying snippet
:loop		movs	:padr0, tmp
		add	tmp, #1
:padr0		mov	ea, 0-0				' fetch the PASM opcode
		shr	ea, #23 WZ, NR			' test for a NOP opcode
	if_nz	jmp	#:loop				' let's hope this cannot fail to find the last PC long
		mov	PC, ea				' this is the PC where to start recompiling at
		jmp	#compile

'*******************************************************
' Handle a return from subroutine
'
handle_ret
		call	#pop_alu
		mov	PC, alu				' and set the new i8085 program counter
		jmp	#compile

'*******************************************************
' Handle an absolute call outside of the compiled space
'
handle_call
		mov	alu, PC
		call	#push_alu
		mov	PC, aux				' set the new i8085 program counter
		jmp	#compile


'*******************************************************
' Write a byte from the alu to RAM[HL]
'
wr_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		jmp	#wr_byte

'*******************************************************
' Write a byte from the alu to RAM[DE]
'
wr_byte_de
		mov	ea, D
		shl	ea, #8
		or	ea, E
		' fall through

'*******************************************************
' Write a byte from the alu to RAM[ea]
'
' If the write modifies the compiled code,
' we need to bail out and recompile from PC
'
wr_byte
		mov	hubptr, ea
		add	hubptr, ram_base
		wrbyte	alu, hubptr
		cmp	ea, PC0	WZ, WC			' writing below PC0 ?
	if_b	jmp	#wr_byte_ret
		cmp	ea, PC WZ, WC			' writing PC or beyond, i.e. end of compiled block ?
	if_a	jmp	#wr_byte_ret
		mov	tmp, wr_byte_ret		' get the return address in the source field
		jmp	#snippet_pc			' need to recompile from PC
wr_byte_ret
wr_byte_de_ret
wr_byte_hl_ret
		ret

'*******************************************************
' Read a byte from RAM[HL] into the alu
'
rd_byte_hl
		mov	ea, H
		shl	ea, #8
		or	ea, L
		' fall through

'*******************************************************
' Read a byte from RAM[ea] into the alu
'
rd_byte
		mov	hubptr, ea
		add	hubptr, ram_base
		rdbyte	alu, hubptr
rd_byte_ret
rd_byte_hl_ret
		ret

'*******************************************************
' Read a byte from RAM[PC] into the opcode and
' increment PC by 1
'
rd_opcode
		mov	hubptr, PC
		add	PC, #1
		and	PC, low_16bits
		add	hubptr, ram_base
		rdbyte	opcode, hubptr
rd_opcode_ret
		ret

'*******************************************************
' Write a word from the alu to RAM[ea]
'
' If the write modifies the compiled code,
' we need to bail out and recompile from PC
'
wr_word
		mov	hubptr, ea
		add	hubptr, ram_base
		wrbyte	alu, hubptr
		add	hubptr, #1
		ror	alu, #8
		wrbyte	alu, hubptr
		rol	alu, #8				' restore alu (FIXME: do we need this?)
		cmp	ea, PC0	WZ, WC			' writing below PC0 ?
	if_b	jmp	#wr_word_ret
		cmp	ea, PC WZ, WC			' writing PC or beyond, i.e. end of compiled block ?
	if_a	jmp	#wr_word_ret
		mov	tmp, wr_word_ret		' get the return address in the source field
		jmp	#snippet_pc			' need to recompile from PC
wr_word_ret
		ret

'*******************************************************
' Read a word from RAM[PC] into the alu and increment
' PC by 2
'
rd_opword
		mov	ea, PC
		add	PC, #2
		and	PC, low_16bits
		' fall through

'*******************************************************
' Read a word from RAM[ea] into the alu
'
rd_word
		mov	hubptr, ea
		add	hubptr, ram_base
		rdbyte	alu, hubptr
		add	hubptr, #1
		ror	alu, #8
		rdbyte	aux, hubptr
		or	alu, aux
		rol	alu, #8
rd_word_ret
rd_opword_ret
		ret

'*******************************************************
' Write a byte from the alu to an I/O port
'
wr_port
		cmp	ea, #CONOUT_PORT WZ
	if_z	jmp	#wr_conout
		mov	hubptr, ea
		add	hubptr, io_base
		wrbyte	alu, hubptr
wr_port_ret
		ret

wr_conout
		rdlong	t1, conout WZ
	if_nz	jmp	#wr_conout
		mov	t1, #$100
		or	t1, alu
		wrlong	t1, conout
		jmp	wr_port_ret

'*******************************************************
' Read a byte from an I/O port into the alu
'
rd_port_alu
		mov	hubptr, ea
		add	hubptr, io_base
		rdbyte	alu, hubptr
rd_port_alu_ret
		ret


'*******************************************************
' Pop a word from the i8085 stack to alu
'
pop_alu
		mov	ea, SP				' pop a word
		add	SP, #2				' off the stack
		and	SP, low_16bits
		call	#rd_word			' into alu
pop_alu_ret
		ret

'*******************************************************
' Push a word from alu to the i8085 stack
'
push_alu
		sub	SP, #2
		and	SP, low_16bits
		mov	ea, SP
		call	#wr_word			' from alu
push_alu_ret
		ret

'******************************************************************************
'******************************************************************************
'
'	Debug code
'
'******************************************************************************
'******************************************************************************
#ifdef	DEBUG
scr_space
		mov	char, #32
scr_chr
		cmp	char, #13 WZ
	if_nz	jmp	#scr_output
scr_cr
		mov	scrx, #SCR_X
		add	scry, #1
		cmp	scry, #(SCR_Y + SCR_H) WC
	if_c	jmp	#scr_cr_ret
		mov	scry, #(SCR_Y + SCR_H - 1)
		call	#scr_scroll
		jmp	#scr_cr_ret
scr_output
		mov	t1, scry		' y
		shl	t1, #2			' * 4
		add	t1, scry		' * 5
		shl	t1, #3			' * 40
		add	t1, scrx		' + x
		add	t1, scr			' + screen base
		wrbyte	char, t1
		add	scrx, #1
		cmp	scrx, #SCR_W WC		' < 40?
	if_nc	mov	scrx, #(SCR_W - 1)
scr_cr_ret
scr_chr_ret
scr_space_ret
		ret

x20202020	long	$20202020
scr_scroll
		mov	t1, #SCR_Y
		shl	t1, #2			' * 4
		add	t1, #SCR_Y		' * 5
		shl	t1, #3			' * 40
		add	t1, scr
		mov	t2, t1
		add	t1, #SCR_W
		mov	t3, #(SCR_H*40/4)
:scroll		rdlong	char, t1
		add	t1, #4
		wrlong	char, t2
		add	t2, #4
		djnz	t3, #:scroll
		mov	t3, #SCR_W/4
:erase_eol	wrlong	x20202020, t2
		add	t2, #4
		djnz	t3, #:erase_eol
scr_scroll_ret
		ret

hexcnt		long	0
hexval		long	0
hex_l
		mov	hexcnt, #8
hex_loop
		rol	hexval, #4
		mov	char, hexval
		and	char, #15
		cmp	char, #10 WZ, WC
	if_b	add	char, #48
	if_ae	add	char, #55
		call	#scr_chr
		djnz	hexcnt, #hex_loop
hex_loop_ret
hex_l_ret
		ret

hex_b
		mov	hexcnt, #2
		shl	hexval, #24
		call	#hex_loop
hex_b_ret
		ret

hex_w
		mov	hexcnt, #4
		shl	hexval, #16
		call	#hex_loop
hex_w_ret
		ret
#endif

'******************************************************************************
'******************************************************************************
'
'	Compiler output
'
'******************************************************************************
'******************************************************************************
ORIGIN					' here is the origin of the compiler output
		fit	$1f0

'******************************************************************************
'
' Compute an 8 bit increment and set the flags
'
alu_inr		long	(alu_inr_end - $ - 1)
		add	alu, #1				' icrement alu
		mov	tmp, alu
		andn	F, #v_flag			' clear negative flag
		and	alu, #$ff WZ, WC
		muxz	F, #z_flag
		muxnc	F, #p_flag			' P flag set to parity
		test	alu, #$80 WZ			' bit #7 set?
		muxz	F, #s_flag			' S flag used to indicate sign
		and	tmp, #$f
		cmp	tmp, #$0 WZ			' lower nibble wrapped to 0?
		muxz	F, #h_flag			' set auxiliary carry
alu_inr_end


'******************************************************************************
'
' Compute an 8 bit decrement and set the flags
'
alu_dcr		long	(alu_dcr_end - $ - 1)
		sub	alu, #1				' decrement alu
		mov	tmp, alu
		or	F, #v_flag			' set V flag
		and	alu, #$ff WZ, WC
		muxz	F, #z_flag			' Z flag set if zero
		muxnc	F, #p_flag			' P flag set to party
		test	alu, #$80 WZ			' bit #7 set?
		muxz	F, #s_flag			' S flag used to indicate sign
		and	tmp, #$f
		cmp	tmp, #$f WZ			' lower nibble wrapped to f?
		muxz	F, #h_flag			' set auxiliary carry
alu_dcr_end

'******************************************************************************
'
' Compute an arithmetic or logic alu operation and set the flags
'
alu_add		long	(alu_add_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #0				' clear all flags and A
		mov	aux, alu
		add	alu, tmp			' add the tmp value
		xor	aux, tmp
		xor	aux, alu
		test	aux, #h_flag WC			' H flag change?
		muxc	F, #h_flag
		test	aux, #$100 WC
		muxc	F, #(p_flag|c_flag)
		test	aux, #$80 WC
	if_c	xor	F, #p_flag
		and	alu, #$FF WZ
		muxz	F, #z_flag
		test	alu, #$80 WZ
		muxnz	F, #s_flag
		mov	A, alu
alu_add_end

alu_adc		long	(alu_adc_end - $ - 1)
		mov	alu, A				' operate on A
		test	F, #c_flag WC			' get the C flag into carry
		mov	F, #0				' clear all flags and A
		mov	aux, alu
		addx	alu, tmp			' add the tmp value with carry
		xor	aux, tmp
		xor	aux, alu
		test	aux, #h_flag WC			' H flag change?
		muxc	F, #h_flag
		test	aux, #$100 WC
		muxc	F, #(p_flag|c_flag)
		test	aux, #$80 WC
	if_c	xor	F, #p_flag
		and	alu, #$FF WZ
		muxz	F, #z_flag
		test	alu, #$80 WZ
		muxnz	F, #s_flag
		mov	A, alu
alu_adc_end

alu_sub		long	(alu_sub_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #v_flag			' clear all flags but the V flag
		mov	aux, alu
		sub	alu, tmp			' subtract the tmp value
		xor	aux, tmp
		xor	aux, alu
		test	aux, #h_flag WC			' H flag change?
		muxc	F, #h_flag
		test	aux, #$100 WC
		muxc	F, #(p_flag|c_flag)
		test	aux, #$80 WC
	if_c	xor	F, #p_flag
		and	alu, #$FF WZ
		muxz	F, #z_flag
		test	alu, #$80 WZ
		muxnz	F, #s_flag
		mov	A, alu
alu_sub_end

alu_sbb		long	(alu_sbb_end - $ - 1)
		mov	alu, A				' operate on A
		test	F, #c_flag WC			' get the C flag into carry
		mov	F, #v_flag			' clear all flags but the V flag
		mov	aux, alu
		subx	alu, tmp			' add the tmp value
		xor	aux, tmp
		xor	aux, alu
		test	aux, #h_flag WC			' H flag change?
		muxc	F, #h_flag
		test	aux, #$100 WC
		muxc	F, #(p_flag|c_flag)
		test	aux, #$80 WC
	if_c	xor	F, #p_flag
		and	alu, #$FF WZ
		muxz	F, #z_flag
		test	alu, #$80 WZ
		muxnz	F, #s_flag
		mov	A, alu
alu_sbb_end

alu_and		long	(alu_and_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #0				' clear flags
		and	alu, tmp WZ, WC			' and with the tmp value
		muxz	F, #z_flag			' if zero set Z flag
		muxnc	F, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	F, #s_flag			' set S flag
		mov	A, alu
alu_and_end

alu_xra		long	(alu_xra_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #0				' clear flags
		xor	alu, tmp WZ, WC			' xor with the tmp value
		muxz	F, #z_flag			' if zero set Z flag
		muxnc	F, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	F, #s_flag			' set S flag
		mov	A, alu
alu_xra_end

alu_ora		long	(alu_ora_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #0				' clear flags
		or	alu, tmp WZ, WC			' or with the tmp value
		muxz	F, #z_flag			' if zero set Z flag
		muxnc	F, #p_flag			' if even parity set the P flag
		test	alu, #$80 WC			' get sign in prop's carry flag
		muxc	F, #s_flag			' set S flag
		mov	A, alu
alu_ora_end

alu_cmp		long	(alu_cmp_end - $ - 1)
		mov	alu, A				' operate on A
		mov	F, #v_flag			' clear all flags but the V flag
		mov	aux, alu
		sub	alu, tmp			' subtract the tmp value
		xor	aux, tmp
		xor	aux, alu
		test	aux, #h_flag WC			' H flag change?
		muxc	F, #h_flag
		test	aux, #$100 WC
		muxc	F, #(p_flag|c_flag)
		test	aux, #$80 WC
	if_c	xor	F, #p_flag
		and	alu, #$FF WZ
		muxz	F, #z_flag
		test	alu, #$80 WZ
		muxnz	F, #s_flag
alu_cmp_end

alu_add16	long	(alu_add16_end - $ - 1)
		mov	alu, H
		shl	alu, #8
		or	alu, L
		mov	aux, alu
		add	alu, tmp
		shl	alu, #16 WC			' get bit #16 into prop's carry
		muxc	F, #c_flag			' clear or set carry flag
		shr	alu, #16
		xor	aux, alu
		xor	aux, tmp
		shr	aux, #8				' get MSB to LSB
		test	aux, #h_flag WC
		muxc	F, #h_flag			' clear or set auxiliary carry flag
		ror	alu, #8				' MSB in 7..0
		movs	H, alu				' move lower 9 bits
		shr	alu, #24			' LSB in 7..0
		mov	L, alu
alu_add16_end

'*******************************************************
' 00 No operation
'
i_nop		long	(:end - $ - 1)
:end

#ifdef	CPU_8085
'*******************************************************
' 08 Subtract DE from HL (8085) FIXME: flags?
'
i_dsub		long	(:end - $ - 1)
		sub	L, E WC
		and	L, #$ff
		subx	H, D
		and	H, #$ff		
:end

'*******************************************************
' 10 Arithmetic shift right HL (8085)
'
i_asrh		long	(:end - $ - 1)
		shr	H, #1 WC
		rcr	L, #1 WC
		muxc	F, #c_flag
:end

'*******************************************************
' 18 Rotate left DE (8085)
'
i_rlde		long	(:end - $ - 1)
		andn	F, #(v_flag|c_flag)
		shl	E, #25 WC
		muxc	F, #c_flag
		shr	E, #24
		mov	aux, D
		rcl	D, #1
		test	D, #$100 WC
		muxc	E, #$01
		test	aux, #$80 WZ
		muxnz	F, #v_flag
	if_c	xor	F, #v_flag
:end

'*******************************************************
' 20 Read interrupt mask (8085)
'
i_rim		long	(:end - $ - 1)
		' not implemented
:end

'*******************************************************
' 28 Load DE with HL + nn (8085)
'
i_ldeh		long	(:end - $ - 1)
		mov	tmp, #0-0		' source is patched with the imm8
		mov	E, L
		add	E, tmp
		test	E, #$100 WC
		and	E, #$ff
		mov	D, H
		addx	D, #0
		and	D, #$ff
:end

'*******************************************************
' 30 Set interrupt mask (8085)
'
i_sim		long	(:end - $ - 1)
		' not implemented
:end

'*******************************************************
' 38 Load DE with SP + nn  (8085)
'
i_ldes		long	(:end - $ - 1)
		mov	tmp, #0-0		' source is patched with the imm8
		mov	E, SP
		add	E, tmp
		mov	D, E
		and	E, #$ff
		shr	D, #8
:end

#else

'*******************************************************
' 08 NOP (undocumented)
'
'i_nop_08	long	(:end - $ - 1)
':end

'*******************************************************
' 10 NOP (undocumented)
'
'i_nop_10	long	(:end - $ - 1)
':end

'*******************************************************
' 18 NOP (undocumented)
'
'i_nop_18	long	(:end - $ - 1)
':end

'*******************************************************
' 20 NOP (undocumented)
'
'i_nop_20	long	(:end - $ - 1)
':end

'*******************************************************
' 28 NOP (undocumented)
'
'i_nop_28	long	(:end - $ - 1)
':end

'*******************************************************
' 30 NOP (undocumented)
'
'i_nop_30	long	(:end - $ - 1)
':end

'*******************************************************
' 38 NOP (undocumented)
'
'i_nop_38	long	(:end - $ - 1)
':end

#endif

'*******************************************************
' 01 Load BC with imm16
'
i_lxi_b_imm16	long	(:end - $ - 1)
		mov	C, 0-0			' source is patched with the address of imm16
		mov	B, C
		and	C, #$ff
		shr	B, #8
:end

'*******************************************************
' 11 Load DE with imm16
'
i_lxi_d_imm16	long	(:end - $ - 1)
		mov	E, 0-0			' source is patched with the address of imm16
		mov	D, E
		and	E, #$ff
		shr	D, #8
:end

'*******************************************************
' 21 Load HL with imm16
'
i_lxi_h_imm16	long	(:end - $ - 1)
		mov	L, 0-0			' source is patched with the address of imm16
		mov	H, L
		and	L, #$ff
		shr	H, #8
:end

'*******************************************************
' 31 Load SP with imm16
'
i_lxi_sp	long	(:end - $ - 1)
		mov	SP, 0-0			' source is patched with the address of imm16
:end

'*******************************************************
' 09 Add BC to HL
'
i_dad_b		long	(:end - $ - 1) + (alu_add16_end - alu_add16 - 1)
		mov	tmp, B
		shl	tmp, #8
		or	tmp, C
		long	@alu_add16		' paste the add16 code snippet here
:end

'*******************************************************
' 19 Add DE to HL
'
i_dad_d		long	(:end - $ - 1) + (alu_add16_end - alu_add16 - 1)
		mov	tmp, D
		shl	tmp, #8
		or	tmp, E
		long	@alu_add16		' paste the add16 code snippet here
:end

'*******************************************************
' 29 Add HL to HL
'
i_dad_h		long	(:end - $ - 1) + (alu_add16_end - alu_add16 - 1)
		mov	tmp, H
		shl	tmp, #8
		or	tmp, L
		long	@alu_add16		' paste the add16 code snippet here
:end

'*******************************************************
' 39 Add SP to HL
'
i_dad_sp	long	(:end - $ - 1) + (alu_add16_end - alu_add16 - 1)
		mov	tmp, SP
		long	@alu_add16		' paste the add16 code snippet here
:end

'*******************************************************
' 02 Load memory (BC) with accu
'
i_stax_b	long	(:end - $ - 1)
		mov	ea, B
		shl	ea, #8
		or	ea, C
		mov	alu, A
		call	#wr_byte
:end

'*******************************************************
' 0a Load accu from memory (BC)
'
i_ldax_b	long	(:end - $ - 1)
		mov	ea, B
		shl	ea, #8
		or	ea, C
		call	#rd_byte
		mov	A, alu
:end

'*******************************************************
' 12 Load memory (DE) with accu
'
i_stax_d	long	(:end - $ - 1)
		mov	alu, A
		call	#wr_byte_de
:end

'*******************************************************
' 1a Load accu from memory (DE)
'
i_ldax_d	long	(:end - $ - 1)
		mov	ea, D
		shl	ea, #8
		or	ea, E
		call	#rd_byte
		mov	A, alu
:end

'*******************************************************
' 22 Load memory (imm16) with HL
'
i_shld_imm16	long	(:end - $ - 1)
		mov	ea, 0-0			' source is patched with the address of imm16
		mov	alu, L
		call	#wr_byte
		add	ea, #1
		mov	alu, H
		call	#wr_byte
:end

'*******************************************************
' 2a Load HL from memory (imm16)
'
i_lhld_imm16	long	(:end - $ - 1)
		mov	ea, 0-0			' source is patched with the address of imm16
		call	#rd_word
		mov	L, alu			' to L
		and	L, #$ff
		mov	H, alu
		shr	H, #8
:end

'*******************************************************
' 32 Load memory (imm16) with A
'
i_stax_imm16	long	(:end - $ - 1)
		mov	ea, 0-0			' source is patched with the address of imm16
		mov	alu, A			' store AF
		call	#wr_byte
:end

'*******************************************************
' 3a Load A from memory (imm16)
'
i_ldax_imm16	long	(:end - $ - 1)
		mov	ea, 0-0			' source is patched with the address of imm16
		call	#rd_byte
		mov	A, alu
:end

#ifdef	CPU_8085
'*******************************************************
' 03 Increment BC
' 8085 Note: X5 indicates overflow to 0000
'
i_inx_b		long	(:end - $ - 1)
		add	C, #1
		test	C, #$100 WC
		and	C, #$ff WZ
	if_c	add	B, #1
		and	B, #$ff
	if_z	and	B, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 0b Decrement BC
' 8085 Note: X5 indicates overflow to ffff
'
i_dcx_b		long	(:end - $ - 1)
		sub	C, #1 WC
		and	C, #$ff WZ
	if_c	sub	B, #1 WC
		and	B, #$ff
	if_z	and	B, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 13 Increment DE
' 8085 Note: X5 indicates overflow to 0000
'
i_inx_d		long	(:end - $ - 1)
		add	E, #1
		test	E, #$100 WC
		and	E, #$ff WZ
	if_c	add	D, #1
		and	D, #$ff
	if_z	and	D, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 1b Decrement DE
' 8085 Note: X5 indicates overflow to ffff
'
i_dcx_d		long	(:end - $ - 1)
		sub	E, #1 WC
		and	E, #$ff WZ
	if_c	sub	D, #1 WC
		and	D, #$ff
	if_z	and	D, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 23 Increment HL
' 8085 Note: X5 indicates overflow to 0000
'
i_inx_h		long	(:end - $ - 1)
		add	L, #1
		test	L, #$100 WC
		and	L, #$ff WZ
	if_c	add	H, #1
		and	H, #$ff
	if_z	and	H, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 2b Decrement HL
' 8085 Note: X5 indicates overflow to ffff
'
i_dcx_h		long	(:end - $ - 1)
		sub	L, #1 WC
		and	L, #$ff WZ
	if_c	sub	H, #1 WC
		and	H, #$ff
	if_z	and	H, B WZ
		muxz	F, #x5_flag
:end

'*******************************************************
' 33 Increment SP
' 8085 Note: X5 indicates overflow to 0000
'
i_inx_sp	long	(:end - $ - 1)
		add	SP, #1
		and	SP, low_16bits WZ
		muxz	AF, #x5_flag
:end

'*******************************************************
' 3b Decrement SP
' 8085 Note: X5 indicates overflow to ffff
'
i_dcx_sp	long	(:end - $ - 1)
		test	SP, SP WZ
		sub	SP, #1
		and	SP, low_16bits
		muxz	AF, #x5_flag
:end

#else

'*******************************************************
' 03 Increment BC
'
i_inx_b		long	(:end - $ - 1)
		add	C, #1
		test	C, #$100 WC
		and	C, #$ff
	if_c	add	B, #1
		and	B, #$ff
:end

'*******************************************************
' 0b Decrement BC
'
i_dcx_b		long	(:end - $ - 1)
		sub	C, #1 WC
		and	C, #$ff WZ
	if_c	sub	B, #1
		and	B, #$ff
:end

'*******************************************************
' 13 Increment DE
'
i_inx_d		long	(:end - $ - 1)
		add	E, #1
		test	E, #$100 WC
		and	E, #$ff
	if_c	add	D, #1
		and	D, #$ff
:end

'*******************************************************
' 1b Decrement DE
'
i_dcx_d		long	(:end - $ - 1)
		sub	E, #1 WC
		and	E, #$ff WZ
	if_c	sub	D, #1
		and	D, #$ff
:end

'*******************************************************
' 23 Increment HL
'
i_inx_h		long	(:end - $ - 1)
		add	L, #1
		test	L, #$100 WC
		and	L, #$ff
	if_c	add	H, #1
		and	H, #$ff
:end

'*******************************************************
' 2b Decrement HL
'
i_dcx_h		long	(:end - $ - 1)
		sub	L, #1 WC
		and	L, #$ff WZ
	if_c	sub	H, #1
		and	H, #$ff
:end

'*******************************************************
' 33 Increment SP
'
i_inx_sp	long	(:end - $ - 1)
		add	SP, #1
		and	SP, low_16bits
:end

'*******************************************************
' 3b Decrement SP
'
i_dcx_sp	long	(:end - $ - 1)
		sub	SP, #1
		and	SP, low_16bits
:end

#endif

'*******************************************************
' 04 Increment B
'
i_inr_b		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, B
		long	@alu_inr
		mov	B, alu
:end

'*******************************************************
' 0c Increment C
'
i_inr_c		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, C
		long	@alu_inr
		mov	C, alu
:end

'*******************************************************
' 14 Increment D
'
i_inr_d		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, D
		long	@alu_inr
		mov	D, alu
:end


'*******************************************************
' 1c Increment E
'
i_inr_e		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, E
		long	@alu_inr
		mov	E, alu
:end

'*******************************************************
' 24 Increment H
'
i_inr_h		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, H
		long	@alu_inr
		mov	H, alu
:end


'*******************************************************
' 2c Increment L
'
i_inr_l		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, L
		long	@alu_inr
		mov	L, alu
:end

'*******************************************************
' 34 Increment memory (HL)
'
i_inr_m		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		call	#rd_byte_hl
		long	@alu_inr
		call	#wr_byte
:end

'*******************************************************
' 3c Increment A
'
i_inr_a		long	(:end - $ - 1) + (alu_inr_end - alu_inr - 1)
		mov	alu, A
		long	@alu_inr
		mov	A, alu
:end

'*******************************************************
' 05 Decrement B
'
i_dcr_b		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, B
		long	@alu_dcr
		mov	B, alu
:end

'*******************************************************
' 0d Decrement C
'
i_dcr_c		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, C
		long	@alu_dcr
		mov	C, alu
:end

'*******************************************************
' 15 Decrement D
'
i_dcr_d		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, D
		long	@alu_dcr
		mov	D, alu
:end

'*******************************************************
' 1d Decrement E
'
i_dcr_e		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, E
		long	@alu_dcr
		mov	E, alu
:end

'*******************************************************
' 25 Decrement H
'
i_dcr_h		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, H
		long	@alu_dcr
		mov	H, alu
:end

'*******************************************************
' 2d Decrement L
'
i_dcr_l		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, L
		long	@alu_dcr
		mov	L, alu
:end

'*******************************************************
' 35 Decrement memory (HL)
'
i_dcr_m		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		call	#rd_byte_hl
		long	@alu_dcr
		call	#wr_byte
:end

'*******************************************************
' 3d Decrement A
'
i_dcr_a		long	(:end - $ - 1) + (alu_dcr_end - alu_dcr - 1)
		mov	alu, A
		long	@alu_dcr
		mov	A, alu
:end

'*******************************************************
' 06 Load B with imm8
'
i_mvi_b_imm8	long	(:end - $ - 1)
		mov	B, #0			' source is patched with the imm8
:end

'*******************************************************
' 0e Load C with imm8
'
i_mvi_c_imm8	long	(:end - $ - 1)
		mov	C, #0			' source is patched with the imm8
:end

'*******************************************************
' 16 Load D with imm8
'
i_mvi_d_imm8	long	(:end - $ - 1)
		mov	D, #0			' source is patched with the imm8
:end

'*******************************************************
' 1e Load E with imm8
'
i_mvi_e_imm8	long	(:end - $ - 1)
		mov	E, #0			' source is patched with the imm8
:end

'*******************************************************
' 26 Load H with imm8
'
i_mvi_h_imm8	long	(:end - $ - 1)
		mov	H, #0			' source is patched with the imm8
:end

'*******************************************************
' 2e Load L with imm8
'
i_mvi_l_imm8	long	(:end - $ - 1)
		mov	L, #0			' source is patched with the imm8
:end

'*******************************************************
' 36 Load memory (HL) with imm8
'
i_mvi_m_imm8	long	(:end - $ - 1)
		mov	alu, #0			' source is patched with the imm8
		call	#wr_byte_hl
:end

'*******************************************************
' 3e Load A with imm8
'
i_mvi_a_imm8	long	(:end - $ - 1)
		mov	A, #0			' source is patched with the imm8
:end

'*******************************************************
' 07 Rotate left circular
'
i_rlc		long	(:end - $ - 1)
		mov	alu, A
		shl	alu, #1
		test	alu, #$100 WC		' check carry
		and	alu, #$ff		' keep bits 7..0
		muxc	F, #c_flag		' set carry flag
		muxc	alu, #$01		' set low bit
		mov	A, alu
:end


'*******************************************************
' 0f Rotate right circular
'
i_rrc		long	(:end - $ - 1)
		mov	alu, A
		shr	alu, #1 WC
		muxc	F, #c_flag		' set carry flag
		muxc	alu, #$80		' set high bit
		mov	A, alu
:end

'*******************************************************
' 17 Rotate accu left through carry
'
i_ral		long	(:end - $ - 1)
		mov	alu, A
		shl	alu, #1
		test	alu, #$100 WZ		' test bit #7
		and	alu, #$ff		' keep bits 7..1
		test	F, #c_flag WC		' get previous C flag
		muxnz	F, #c_flag		' set carry flag
		muxc	alu, #$01		' set bit #0 from previous C flag
		mov	A, alu
:end

'*******************************************************
' 1f Rotate accu right through carry
'
i_rar		long	(:end - $ - 1)
		mov	alu, A
		shr	alu, #1 WC		' get bit #0 into carry
		test	F, #c_flag WZ		' get previous C flag
		muxc	F, #c_flag		' set C from carry flag
		muxnz	alu, #$80		' set bit #7 from previous C flag
		mov	A, alu
:end

'*******************************************************
' 27 Decimal adjust accumulator
'
i_daa		long	(:end - $ - 1)
		mov	alu, A
		mov	tmp, alu
		mov	aux, alu		' save for auxiliary flag test
		and	tmp, #15		' isolate low nibble of accumulator
		mov	adj_06, #$06
		mov	adj_60, #$60
		test	F, #v_flag WZ		' test negative flag
		negnz	adj_06, adj_06
		negnz	adj_60, adj_60
		test	F, #h_flag WZ
		cmp	tmp, #$0a WC		' aux isn't set; is nibble > $9 ?
  if_nc_or_nz	add	alu, adj_06		' aux was set or nibble is > $9 => add $06 or -$06
		test	F, #c_flag WZ
		cmp	alu, #$9a WC		' carry isn't set; is data > $99 ?
  if_nc_or_nz	add	alu, adj_60		' carry was set or data is > $99 => add $60 or -$60
		and	F, #(v_flag|c_flag)	' preserve old CF and NF
		cmp	alu, #$9a WC
	if_nc	or	F, #c_flag		' set carry if result > $99
		xor	aux, alu
		and	aux, #h_flag		' if bit #4 of result changed
		or	F, aux			' set the auxiliary carry flag
		and	alu, #$ff WZ, WC
		muxz	F, #z_flag		' set i8085 zero flag from props zero
		muxnc	F, #p_flag		' set i8085 parity flag from props carry
		test	alu, #$80 WZ
		muxnz	F, #s_flag
		mov	A, alu
:end

'*******************************************************
' 2f Complement accu
'
i_cma		long	(:end - $ - 1)
		xor	A, #$ff
		or	F, #(h_flag|v_flag)	' set the H and V flags
:end

'*******************************************************
' 37 Set carry flag
'
i_stc		long	(:end - $ - 1)
		or	F, #c_flag		' set the C flag
:end

'*******************************************************
' 3f Complement carry flag
'
i_cmc		long	(:end - $ - 1)
		mov	aux, A
		test	F, #c_flag WC		' get the C flag into the carry
		muxc	F, #h_flag		' set the H flag if C was set
		muxnc	F, #c_flag		' toggle the C flag
		and	aux, #(x5_flag | x3_flag)
		or	F, aux			' or in the X5 and X3 flags
:end


'*******************************************************
' 40 Load B with B (NOP)
'
'i_mov_b_b	long	(:end - $ - 1)
':end

'*******************************************************
' 41 Load B with C
'
i_mov_b_c	long	(:end - $ - 1)
		mov	B, C
:end

'*******************************************************
' 42 Load B with D
'
i_mov_b_d	long	(:end - $ - 1)
		mov	B, D
:end

'*******************************************************
' 43 Load B with E
'
i_mov_b_e	long	(:end - $ - 1)
		mov	B, E
:end

'*******************************************************
' 44 Load B with H
'
i_mov_b_h	long	(:end - $ - 1)
		mov	B, H
:end

'*******************************************************
' 45 Load B with L
'
i_mov_b_l	long	(:end - $ - 1)
		mov	B, L
:end

'*******************************************************
' 46 Load B with memory (HL)
'
i_mov_b_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	B, alu
:end

'*******************************************************
' 47 Load B with A
'
i_mov_b_a	long	(:end - $ - 1)
		mov	B, A
:end

'*******************************************************
' 48 Load C with B
'
i_mov_c_b	long	(:end - $ - 1)
		mov	C, B
:end

'*******************************************************
' 49 Load C with C (NOP)
'
'i_mov_c_c	long	(:end - $ - 1)
':end

'*******************************************************
' 4a Load C with D
'
i_mov_c_d	long	(:end - $ - 1)
		mov	C, D
:end

'*******************************************************
' 4b Load C with E
'
i_mov_c_e	long	(:end - $ - 1)
		mov	C, E
:end

'*******************************************************
' 4c Load B with H
'
i_mov_c_h	long	(:end - $ - 1)
		mov	C, H
:end

'*******************************************************
' 4d Load C with L
'
i_mov_c_l	long	(:end - $ - 1)
		mov	C, L
:end

'*******************************************************
' 4e Load C with memory (HL)
'
i_mov_c_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	C, alu
:end

'*******************************************************
' 4f Load C with A
'
i_mov_c_a	long	(:end - $ - 1)
		mov	C, A
:end

'*******************************************************
' 50 Load D with B
'
i_mov_d_b	long	(:end - $ - 1)
		mov	D, B
:end

'*******************************************************
' 51 Load D with C
'
i_mov_d_c	long	(:end - $ - 1)
		mov	D, C
:end

'*******************************************************
' 52 Load D with D (NOP)
'
'i_mov_d_d	long	(:end - $ - 1)
':end

'*******************************************************
' 53 Load D with E
'
i_mov_d_e	long	(:end - $ - 1)
		mov	D, E
:end

'*******************************************************
' 54 Load D with H
'
i_mov_d_h	long	(:end - $ - 1)
		mov	D, H
:end

'*******************************************************
' 55 Load D with L
'
i_mov_d_l	long	(:end - $ - 1)
		mov	D, L
:end

'*******************************************************
' 56 Load D with memory (HL)
'
i_mov_d_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	D, alu
:end

'*******************************************************
' 57 Load D with A
'
i_mov_d_a	long	(:end - $ - 1)
		mov	D, A
:end

'*******************************************************
' 58 Load E with B
'
i_mov_e_b	long	(:end - $ - 1)
		mov	E, B
:end

'*******************************************************
' 59 Load E with C
'
i_mov_e_c	long	(:end - $ - 1)
		mov	E, C
:end

'*******************************************************
' 5a Load E with D
'
i_mov_e_d	long	(:end - $ - 1)
		mov	E, D
:end

'*******************************************************
' 5b Load E with E (NOP)
'
'i_mov_e_e	long	(:end - $ - 1)
':end

'*******************************************************
' 5c Load E with H
'
i_mov_e_h	long	(:end - $ - 1)
		mov	E, H
:end

'*******************************************************
' 5d Load E with L
'
i_mov_e_l	long	(:end - $ - 1)
		mov	E, L
:end

'*******************************************************
' 5e Load E with memory (HL)
'
i_mov_e_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	E, alu
:end

'*******************************************************
' 5f Load E with A
'
i_mov_e_a	long	(:end - $ - 1)
		mov	E, A
:end

'*******************************************************
' 60 Load H with B
'
i_mov_h_b	long	(:end - $ - 1)
		mov	H, B
:end

'*******************************************************
' 61 Load H with C
'
i_mov_h_c	long	(:end - $ - 1)
		mov	H, C
:end

'*******************************************************
' 62 Load H with D
'
i_mov_h_d	long	(:end - $ - 1)
		mov	H, D
:end

'*******************************************************
' 63 Load H with E
'
i_mov_h_e	long	(:end - $ - 1)
		mov	H, E
:end

'*******************************************************
' 64 Load H with H (NOP)
'
'i_mov_h_h	long	(:end - $ - 1)
':end

'*******************************************************
' 65 Load H with L
'
i_mov_h_l	long	(:end - $ - 1)
		mov	H, L
:end

'*******************************************************
' 66 Load H with memory (HL)
'
i_mov_h_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	H, alu
:end

'*******************************************************
' 67 Load H with A
'
i_mov_h_a	long	(:end - $ - 1)
		mov	H, A
:end

'*******************************************************
' 68 Load L with B
'
i_mov_l_b	long	(:end - $ - 1)
		mov	L, B
:end

'*******************************************************
' 69 Load L with C
'
i_mov_l_c	long	(:end - $ - 1)
		mov	L, C
:end

'*******************************************************
' 6a Load L with D
'
i_mov_l_d	long	(:end - $ - 1)
		mov	L, D
:end

'*******************************************************
' 6b Load L with E
'
i_mov_l_e	long	(:end - $ - 1)
		mov	L, E
:end

'*******************************************************
' 6c Load L with H
'
i_mov_l_h	long	(:end - $ - 1)
		mov	L, H
:end

'*******************************************************
' 6d Load L with L (NOP)
'
'i_mov_l_l	long	(:end - $ - 1)
':end

'*******************************************************
' 6e Load L with memory (HL)
'
i_mov_l_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	L, alu
:end

'*******************************************************
' 6f Load L with A
'
i_mov_l_a	long	(:end - $ - 1)
		mov	L, A
:end

'*******************************************************
' 70 Load memory (HL) with B
'
i_mov_m_b	long	(:end - $ - 1)
		mov	alu, B
		call	#wr_byte_hl
:end

'*******************************************************
' 71 Load memory (HL) with C
'
i_mov_m_c	long	(:end - $ - 1)
		mov	alu, C
		call	#wr_byte_hl
:end

'*******************************************************
' 72 Load memory (HL) with D
'
i_mov_m_d	long	(:end - $ - 1)
		mov	alu, D
		call	#wr_byte_hl
:end

'*******************************************************
' 73 Load memory (HL) with E
'
i_mov_m_e	long	(:end - $ - 1)
		mov	alu, E
		call	#wr_byte_hl
:end

'*******************************************************
' 74 Load memory (HL) with H
'
i_mov_m_h	long	(:end - $ - 1)
		mov	alu, H
		call	#wr_byte_hl
:end

'*******************************************************
' 75 Load memory (HL) with L
'
i_mov_m_l	long	(:end - $ - 1)
		mov	alu, L
		call	#wr_byte_hl
:end

'*******************************************************
' 76 HALT
'
i_halt		long	(:end - $ - 1)
		mov	halt, #1		' set the halt flag
:end

'*******************************************************
' 77 Load memory (HL) with A
'
i_mov_m_a	long	(:end - $ - 1)
		mov	alu, A
		call	#wr_byte_hl
:end

'*******************************************************
' 78 Load A with B
'
i_mov_a_b	long	(:end - $ - 1)
		mov	A, B
:end

'*******************************************************
' 79 Load A with C
'
i_mov_a_c	long	(:end - $ - 1)
		mov	A, C
:end

'*******************************************************
' 7a Load A with D
'
i_mov_a_d	long	(:end - $ - 1)
		mov	A, D
:end

'*******************************************************
' 7b Load A with E
'
i_mov_a_e	long	(:end - $ - 1)
		mov	A, E
:end

'*******************************************************
' 7c Load A with H
'
i_mov_a_h	long	(:end - $ - 1)
		mov	A, H
:end

'*******************************************************
' 7d Load A with L
'
i_mov_a_l	long	(:end - $ - 1)
		mov	A, L
:end

'*******************************************************
' 7e Load A with memory (HL)
'
i_mov_a_m	long	(:end - $ - 1)
		call	#rd_byte_hl
		mov	A, alu
:end

'*******************************************************
' 7f Load A with A (NOP)
'
'i_mov_a_a	long	(:end - $ - 1)
':end

'*******************************************************
' 80 Add B to A
'
i_add_b		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, B
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 81 Add C to A
'
i_add_c		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, C
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 82 Add D to A
'
i_add_d		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, D
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 83 Add E to A
'
i_add_e		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, E
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 84 Add H to A
'
i_add_h		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, H
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 85 Add L to A
'
i_add_l		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, L
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 86 Add memory (HL) to A
'
i_add_m		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 87 Add A to A
'
i_add_a		long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, A
		long	@alu_add		' paste the alu_add code snippet here
:end

'*******************************************************
' 88 Add with carry B to A
'
i_adc_b		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, B
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 89 Add with carry C to A
'
i_adc_c		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, C
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8a Add with carry D to A
'
i_adc_d		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, D
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8b Add with carry E to A
'
i_adc_e		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, E
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8c Add with carry H to A
'
i_adc_h		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, H
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8d Add with carry L to A
'
i_adc_l		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, L
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8e Add with carry memory (HL) to A
'
i_adc_m		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 8f Add with carry A to A
'
i_adc_a		long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, A
		long	@alu_adc		' paste the alu_adc code snippet here
:end

'*******************************************************
' 90 Subtract B from A 
'
i_sub_b		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, B
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 91 Subtract C from A 
'
i_sub_c		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, C
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 92 Subtract D from A 
'
i_sub_d		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, D
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 93 Subtract E from A 
'
i_sub_e		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, E
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 94 Subtract H from A 
'
i_sub_h		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, H
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 95 Subtract L from A 
'
i_sub_l		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, L
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 96 Subtract memory (HL) from A 
'
i_sub_m		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 97 Subtract A from A 
'
i_sub_a		long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, A
		long	@alu_sub		' paste the alu_sub code snippet here
:end

'*******************************************************
' 98 Subtract with carry B from A 
'
i_sbb_b		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, B
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 99 Subtract with carry C from A 
'
i_sbb_c		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, C
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9a Subtract with carry D from A 
'
i_sbb_d		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, D
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9b Subtract with carry E from A 
'
i_sbb_e		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, E
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9c Subtract with carry H from A 
'
i_sbb_h		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, H
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9d Subtract with carry L from A 
'
i_sbb_l		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, L
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9e Subtract with carry memory (HL) from A 
'
i_sbb_m		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' 9f Subtract with carry A from A 
'
i_sbb_a		long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, A
		long	@alu_sbb		' paste the alu_sbb code snippet here
:end

'*******************************************************
' a0 And A with B
'
i_ana_b		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, B
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a1 And A with C
'
i_ana_c		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, C
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a2 And A with D
'
i_ana_d		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, D
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a3 And A with E
'
i_ana_e		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, E
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a4 And A with H
'
i_ana_h		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, H
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a5 And A with L
'
i_ana_l		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, L
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a6 And A with memory (HL)
'
i_ana_m		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a7 And A with A
'
i_ana_a		long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, A
		long	@alu_and		' paste the alu_and code snippet here
:end

'*******************************************************
' a8 Exclusive or A with B
'
i_xra_b		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, B
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' a9 Exclusive or A with C
'
i_xra_c		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, C
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' aa Exclusive or A with D
'
i_xra_d		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, D
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' ab Exclusive or A with E
'
i_xra_e		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, E
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' ac Exclusive or A with H
'
i_xra_h		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, H
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' ad Exclusive or A with L
'
i_xra_l		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, L
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' ae Exclusive or A with memory (HL)
'
i_xra_m		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' af Exclusive or A with A
'
i_xra_a		long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, A
		long	@alu_xra		' paste the alu_xra code snippet here
:end

'*******************************************************
' b0 Or A with B
'
i_ora_b		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, B
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b1 Or A with C
'
i_ora_c		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, C
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b2 Or A with D
'
i_ora_d		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, D
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b3 Or A with E
'
i_ora_e		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, E
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b4 Or A with H
'
i_ora_h		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, H
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b5 Or A with L
'
i_ora_l		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, L
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b6 Or A with memory (HL)
'
i_ora_m		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b7 Or A with A
'
i_ora_a		long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, A
		long	@alu_ora			' paste the alu_ora code snippet here
:end

'*******************************************************
' b8 Compare A with B
'
i_cmp_b		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, B
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' b9 Compare A with C
'
i_cmp_c		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, C
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' ba Compare A with D
'
i_cmp_d		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, D
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' bb Compare A with E
'
i_cmp_e		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, E
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' bc Compare A with H
'
i_cmp_h		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, H
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' bd Compare A with L
'
i_cmp_l		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, L
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' be Compare A with memory (HL)
'
i_cmp_m		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		call	#rd_byte_hl
		mov	tmp, alu
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' bf Compare A with A
'
i_cmp_a		long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, A
		long	@alu_cmp			' paste the alu_cmp code snippet here
:end

'*******************************************************
' c0 Return on Z flag clear (non zero)
'
i_rnz		long	(:end - $ - 1)
		test	F, #z_flag WC		' get Z flag into carry
	if_nc	jmp	#handle_ret		' do the return if Z is clear
:end

'*******************************************************
' c8 Return on Z flag set (zero)
'
i_rz		long	(:end - $ - 1)
		test	F, #z_flag WC		' get Z flag into carry
	if_c	jmp	#handle_ret		' do the return if Z is set
:end

'*******************************************************
' d0 Return on C flag clear (non carry)
'
i_rnc		long	(:end - $ - 1)
		test	F, #c_flag WC		' get C flag into carry
	if_nc	jmp	#handle_ret		' do the return if C is clear
:end

'*******************************************************
' d8 Return on C flag set (carry)
'
i_rc		long	(:end - $ - 1)
		test	F, #c_flag WC		' get C flag into carry
	if_c	jmp	#handle_ret		' do the return if C is set
:end

'*******************************************************
' e0 Return on P flag clear (parity odd)
'
i_rpo		long	(:end - $ - 1)
		test	F, #p_flag WC		' get P flag into carry
	if_nc	jmp	#handle_ret		' do the return if P is clear
:end

'*******************************************************
' e8 Return on P flag set (parity even)
'
i_rpe		long	(:end - $ - 1)
		test	F, #p_flag WC		' get P flag into carry
	if_c	jmp	#handle_ret		' do the return if P is set
:end

'*******************************************************
' f0 Return on S flag clear (plus)
'
i_rp		long	(:end - $ - 1)
		test	F, #s_flag WC		' get S flag into carry
	if_nc	jmp	#handle_ret		' do the return if S is clear
:end

'*******************************************************
' e8 Return on S flag set (minus)
'
i_rm		long	(:end - $ - 1)
		test	F, #s_flag WC		' get S flag into carry
	if_c	jmp	#handle_ret		' do the return if S is set
:end

'*******************************************************
' c1 Pop BC off the stack
'
i_pop_b		long	(:end - $ - 1)
		call	#pop_alu
		mov	C, alu
		and	C, #$ff
		mov	B, alu
		shr	B, #8
:end

'*******************************************************
' d1 Pop DE off the stack
'
i_pop_d		long	(:end - $ - 1)
		call	#pop_alu
		mov	E, alu
		and	E, #$ff
		mov	D, alu
		shr	D, #8
:end

'*******************************************************
' e1 Pop HL off the stack
'
i_pop_h		long	(:end - $ - 1)
		call	#pop_alu
		mov	L, alu
		and	L, #$ff
		mov	H, alu
		shr	H, #8
:end

'*******************************************************
' f1 Pop AF off the stack
'
i_pop_a		long	(:end - $ - 1)
		call	#pop_alu
		mov	F, alu
		and	F, #$ff
		mov	A, alu
		shr	A, #8
:end

'*******************************************************
' c9 Return unconditionally
'
i_ret		long	(:end - $ - 1)
		jmp	#handle_ret		' do the return, i.e. recompile from new PC
:end

#ifdef	CPU_8085
'*******************************************************
' d9 Store HL extended
'
i_shlx		long	(:end - $ - 1)
		mov	alu, L
		call	#wr_byte_de
		add	ea, #1
		mov	alu, H
		call	#wr_byte
:end
#endif

'*******************************************************
' e9 Jump to address in HL
'
i_pchl		long	(:end - $ - 1)
		mov	PC, H
		shl	PC, #8
		or	PC, L
		jmp	#compile
:end

'*******************************************************
' f9 Load SP with HL
'
i_sphl	long	(:end - $ - 1)
		mov	SP, H
		shl	SP, #8
		or	SP, L
:end

'*******************************************************
' c2 Jump on Z flag clear (non zero)
'
i_jnz		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0    		' source may be patched with the destination address if inside PC0..PC
		test	F, #z_flag WC		' get Z flag into carry
	if_nc	tjnz	ea, ea			' jump to already compiled code
	if_nc	mov	PC, aux
	if_nc	jmp	#compile
:end

'*******************************************************
' ca Jump on Z flag set (zero)
'
i_jz		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #z_flag WC		' get Z flag into carry
	if_c	tjnz	ea, ea			' jump to already compiled code
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end

'*******************************************************
' d2 Jump on C flag clear (non carry)
'
i_jnc		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #c_flag WC		' get C flag into carry
	if_nc	tjnz	ea, ea			' jump to already compiled code
	if_nc	mov	PC, aux
	if_nc	jmp	#compile
:end

'*******************************************************
' da Jump on C flag set (carry)
'
i_jc		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #c_flag WC		' get C flag into carry
	if_c	tjnz	ea, ea			' jump to already compiled code
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end

'*******************************************************
' e2 Jump on P flag clear (parity odd)
'
i_jpo		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #p_flag WC		' get P flag into carry
	if_nc	tjnz	ea, ea			' jump to already compiled code
	if_nc	mov	PC, aux
	if_nc	jmp	#compile
:end

'*******************************************************
' ea Jump on P flag set (parity even)
'
i_jpe		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #p_flag WC		' get P flag into carry
	if_c	tjnz	ea, ea			' jump to already compiled code
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end

'*******************************************************
' f2 Jump on S flag clear (plus)
'
i_jp		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #s_flag WC		' get S flag into carry
	if_nc	tjnz	ea, ea			' jump to already compiled code
	if_nc	mov	PC, aux
	if_nc	jmp	#compile
:end

'*******************************************************
' ca Jump on S flag set (minus)
'
i_jm		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		test	F, #s_flag WC		' get S flag into carry
	if_c	tjnz	ea, ea			' jump to already compiled code
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end

'*******************************************************
' c3 Jump unconditionally
'
i_jmp		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of the imm16
		mov	ea, #0-0		' source may be patched with the destination address if inside PC0..PC
		tjnz	ea, ea			' NZ then jump to already compiled code
		mov	PC, aux
		jmp	#compile
:end

#ifdef	CPU_8085
'*******************************************************
' cb Restart on overflow (8085)
'
i_rst_v		long	(:end - $ - 1)
		mov	aux, #ADDR_RSTV		' vector for RST V
		test	F, #v_flag WC
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end
#endif

'*******************************************************
' d3 Output A to port imm8
'
i_out_port_a	long	(:end - $ - 1)
		mov	ea, #0-0		' source is patched with the imm8
		mov	alu, A
		call	#wr_port
:end

'*******************************************************
' db Input A from port imm8
'
i_in_a_port	long	(:end - $ - 1)
		mov	ea, #0-0		' source is patched with the imm8
		call	#rd_port_alu
		mov	A, alu
:end

'*******************************************************
' e3 Exchange stack contents with HL
'
i_xthl		long	(:end - $ - 1)
		mov	ea, SP
		call	#rd_byte
		xor	L, alu
		xor	alu, L
		xor	L, alu
		call	#wr_byte
		add	ea, #1
		call	#rd_byte
		xor	H, alu
		xor	alu, H
		xor	H, alu
		call	#wr_byte
:end

'*******************************************************
' eb Exchange DE with HL
'
i_xchg		long	(:end - $ - 1)
		xor	L, E
		xor	E, L
		xor	L, E
		xor	H, D
		xor	D, H
		xor	H, D
:end

'*******************************************************
' f3 Disable interrupts
'
i_di		long	(:end - $ - 1)
		' nothing to do - yet
:end

'*******************************************************
' fb Enable interrupts
'
i_ei		long	(:end - $ - 1)
		' nothing to do - yet
:end

'*******************************************************
' c4 Call on Z flag clear (non zero)
'
i_cnz		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #z_flag WC		' get Z flag into carry
	if_nc	jmp	#handle_call		' do the call if Z is clear
:end

'*******************************************************
' cc Call on Z flag set (zero)
'
i_cz		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #z_flag WC		' get Z flag into carry
	if_c	jmp	#handle_call		' do the call if Z is set
:end

'*******************************************************
' d4 Call on C flag clear (non carry)
'
i_cnc		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #c_flag WC		' get C flag into carry
	if_nc	jmp	#handle_call		' do the call if C is clear
:end

'*******************************************************
' dc Call on C flag set (carry)
'
i_cc		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #c_flag WC		' get C flag into carry
	if_c	jmp	#handle_call		' do the call if C is set
:end

'*******************************************************
' e4 Call on P flag clear (odd parity)
'
i_cpo		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #p_flag WC		' get P flag into carry
	if_nc	jmp	#handle_call		' do the call if P is clear
:end

'*******************************************************
' cc Call on P flag set (even parity)
'
i_cpe		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #p_flag WC		' get P flag into carry
	if_c	jmp	#handle_call		' do the call if P is set
:end

'*******************************************************
' f4 Call on S flag clear (plus)
'
i_cp		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #s_flag WC		' get S flag into carry
	if_nc	jmp	#handle_call		' do the call if S is clear
:end

'*******************************************************
' fc Call on S flag set (minus)
'
i_cm		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #s_flag WC		' get S flag into carry
	if_c	jmp	#handle_call		' do the call if S is set
:end

'*******************************************************
' c5 Push BC to the stack
'
i_push_b	long	(:end - $ - 1)
		mov	alu, B
		shl	alu, #8
		or	alu, C
		call	#push_alu
:end

'*******************************************************
' d5 Push DE to the stack
'
i_push_d	long	(:end - $ - 1)
		mov	alu, D
		shl	alu, #8
		or	alu, E
		call	#push_alu
:end

'*******************************************************
' e5 Push HL to the stack
'
i_push_h	long	(:end - $ - 1)
		mov	alu, H
		shl	alu, #8
		or	alu, L
		call	#push_alu
:end

'*******************************************************
' f5 Push AF to the stack
'
i_push_a	long	(:end - $ - 1)
		mov	alu, A
		shl	alu, #8
		or	alu, F
		call	#push_alu
:end

'*******************************************************
' cd Call unconditional
'
i_call		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		jmp	#handle_call		' do the call
:end

#ifdef	CPU_8085
'*******************************************************
' dd Jump on overflow clear (8085)
'
i_jnx		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #x5_flag
	if_nc	mov	PC, aux
	if_nc	jmp	#compile
:end

'*******************************************************
' ed Load HL extended (8085)
'
i_lhlx		long	(:end - $ - 1)
		mov	ea, D
		shl	ea, #8
		or	ea, E
		call	#rd_byte
		mov	L, alu
		add	ea, #1
		call	#rd_byte
		mov	H, alu
:end

'*******************************************************
' fd Jump on overflow set (8085)
'
i_jx		long	(:end - $ - 1)
		mov	aux, 0-0		' source is patched with the address of imm16
		test	F, #x5_flag
	if_c	mov	PC, aux
	if_c	jmp	#compile
:end
#endif

'*******************************************************
' c6 Add immediate 8bit to A
'
i_adi_imm8	long	(:end - $ - 1) + (alu_add_end - alu_add - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_add
:end

'*******************************************************
' ce Add with carry immediate 8bit to A
'
i_aci_imm8	long	(:end - $ - 1) + (alu_adc_end - alu_adc - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_adc
:end

'*******************************************************
' d6 Subtract immediate 8bit from A
'
i_sui_imm8	long	(:end - $ - 1) + (alu_sub_end - alu_sub - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_sub
:end

'*******************************************************
' de Subtract with carry immediate 8bit from A
'
i_sbi_imm8	long	(:end - $ - 1) + (alu_sbb_end - alu_sbb - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_sbb
:end

'*******************************************************
' e6 And A with immediate 8bit
'
i_ani_imm8	long	(:end - $ - 1) + (alu_and_end - alu_and - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_and
:end

'*******************************************************
' ee Exclusive or A with immediate 8bit
'
i_xri_imm8	long	(:end - $ - 1) + (alu_xra_end - alu_xra - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_xra
:end

'*******************************************************
' f6 Or A with immediate 8bit
'
i_ori_imm8	long	(:end - $ - 1) + (alu_ora_end - alu_ora - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_ora
:end

'*******************************************************
' fe Compare A with immediate 8bit
'
i_cmi_imm8	long	(:end - $ - 1) + (alu_cmp_end - alu_cmp - 1)
		mov	tmp, #0			' source is patched with the imm8
		long	@alu_cmp
:end

'*******************************************************
' c7 Restart at vector $00
'
i_rst_0		long	(:end - $ - 1)
		mov	aux, #$00
		jmp	#handle_call
:end

'*******************************************************
' cf Restart at vector $08
'
i_rst_1		long	(:end - $ - 1)
		mov	aux, #$08
		jmp	#handle_call
:end

'*******************************************************
' d7 Restart at vector $10
'
i_rst_2		long	(:end - $ - 1)
		mov	aux, #$10
		jmp	#handle_call
:end

'*******************************************************
' df Restart at vector $18
'
i_rst_3		long	(:end - $ - 1)
		mov	aux, #$18
		jmp	#handle_call
:end

'*******************************************************
' e7 Restart at vector $20
'
i_rst_4		long	(:end - $ - 1)
		mov	aux, #$20
		jmp	#handle_call
:end

'*******************************************************
' ef Restart at vector $28
'
i_rst_5		long	(:end - $ - 1)
		mov	aux, #$28
		jmp	#handle_call
:end

'*******************************************************
' f7 Restart at vector $30
'
i_rst_6		long	(:end - $ - 1)
		mov	aux, #$30
		jmp	#handle_call
:end

'*******************************************************
' ff Restart at vector $38
'
i_rst_7		long	(:end - $ - 1)
		mov	aux, #$38
		jmp	#handle_call
:end

snippets
	{00}	long	@i_nop
	{01}	long	@i_lxi_b_imm16	| (patch_imm16 << PATCH_SHIFT)
	{02}	long	@i_stax_b
	{03}	long	@i_inx_b
	{04}	long	@i_inr_b
	{05}	long	@i_dcr_b
	{06}	long	@i_mvi_b_imm8	| (patch_imm8 << PATCH_SHIFT)
	{07}	long	@i_rlc
#ifdef	CPU_8085
	{08}	long	@i_dsub
#else
	{08}	long	@i_nop
#endif
	{09}	long	@i_dad_b
	{0a}	long	@i_ldax_b
	{0b}	long	@i_dcx_b
	{0c}	long	@i_inr_c
	{0d}	long	@i_dcr_c
	{0e}	long	@i_mvi_c_imm8	| (patch_imm8 << PATCH_SHIFT)
	{0f}	long	@i_rrc

#ifdef	CPU_8085
	{10}	long	@i_asrh
#else
	{08}	long	@i_nop
#endif
	{11}	long	@i_lxi_d_imm16	| (patch_imm16 << PATCH_SHIFT)
	{12}	long	@i_stax_d
	{13}	long	@i_inx_d
	{14}	long	@i_inr_d
	{15}	long	@i_dcr_d
	{16}	long	@i_mvi_d_imm8	| (patch_imm8 << PATCH_SHIFT)
	{17}	long	@i_ral
#ifdef	CPU_8085
	{18}	long	@i_rlde
#else
	{08}	long	@i_nop
#endif
	{19}	long	@i_dad_d
	{1a}	long	@i_ldax_d
	{1b}	long	@i_dcx_d
	{1c}	long	@i_inr_e
	{1d}	long	@i_dcr_e
	{1e}	long	@i_mvi_e_imm8	| (patch_imm8 << PATCH_SHIFT)
	{1f}	long	@i_rar

#ifdef	CPU_I8085
	{20}	long	@i_rim
#else
	{08}	long	@i_nop
#endif
	{21}	long	@i_lxi_h_imm16	| (patch_imm16 << PATCH_SHIFT)
	{22}	long	@i_shld_imm16	| (patch_imm16 << PATCH_SHIFT)
	{23}	long	@i_inx_h
	{24}	long	@i_inr_h
	{25}	long	@i_dcr_h
	{26}	long	@i_mvi_h_imm8	| (patch_imm8 << PATCH_SHIFT)
	{27}	long	@i_daa
#ifdef	CPU_I8085
	{28}	long	@i_ldeh		| (patch_imm8 << PATCH_SHIFT)
#else
	{08}	long	@i_nop
#endif
	{29}	long	@i_dad_h
	{2a}	long	@i_lhld_imm16	| (patch_imm16 << PATCH_SHIFT)
	{2b}	long	@i_dcx_h
	{2c}	long	@i_inr_l
	{2d}	long	@i_dcr_l
	{2e}	long	@i_mvi_l_imm8	| (patch_imm8 << PATCH_SHIFT)
	{2f}	long	@i_cma

#ifdef	CPU_I8085
	{30}	long	@i_sim
#else
	{08}	long	@i_nop
#endif
	{31}	long	@i_lxi_sp	| (patch_imm16 << PATCH_SHIFT)
	{32}	long	@i_stax_imm16	| (patch_imm16 << PATCH_SHIFT)
	{33}	long	@i_inx_sp
	{34}	long	@i_inr_m
	{35}	long	@i_dcr_m
	{36}	long	@i_mvi_m_imm8	| (patch_imm8 << PATCH_SHIFT)
	{37}	long	@i_stc
#ifdef	CPU_I8085
	{38}	long	@i_ldes
#else
	{08}	long	@i_nop
#endif
	{39}	long	@i_dad_sp
	{3a}	long	@i_ldax_imm16	| (patch_imm16 << PATCH_SHIFT)
	{3b}	long	@i_dcx_sp
	{3c}	long	@i_inr_a
	{3d}	long	@i_dcr_a
	{3e}	long	@i_mvi_a_imm8	| (patch_imm8 << PATCH_SHIFT)
	{3f}	long	@i_cmc

	{40}	long	@i_nop		' i_mov_b_b
	{41}	long	@i_mov_b_c
	{42}	long	@i_mov_b_d
	{43}	long	@i_mov_b_e
	{44}	long	@i_mov_b_h
	{45}	long	@i_mov_b_l
	{46}	long	@i_mov_b_m
	{47}	long	@i_mov_b_a
	{48}	long	@i_mov_c_b
	{49}	long	@i_nop		' i_mov_c_c
	{4a}	long	@i_mov_c_d
	{4b}	long	@i_mov_c_e
	{4c}	long	@i_mov_c_h
	{4d}	long	@i_mov_c_l
	{4e}	long	@i_mov_c_m
	{4f}	long	@i_mov_c_a

	{50}	long	@i_mov_d_b
	{51}	long	@i_mov_d_c
	{52}	long	@i_nop		' i_mov_d_d
	{53}	long	@i_mov_d_e
	{54}	long	@i_mov_d_h
	{55}	long	@i_mov_d_l
	{56}	long	@i_mov_d_m
	{57}	long	@i_mov_d_a
	{58}	long	@i_mov_e_b
	{59}	long	@i_mov_e_c
	{5a}	long	@i_mov_e_d
	{5b}	long	@i_nop		' i_mov_e_e
	{5c}	long	@i_mov_e_h
	{5d}	long	@i_mov_e_l
	{5e}	long	@i_mov_e_m
	{5f}	long	@i_mov_e_a

	{60}	long	@i_mov_h_b
	{61}	long	@i_mov_h_c
	{62}	long	@i_mov_h_d
	{63}	long	@i_mov_h_e
	{64}	long	@i_nop		' i_mov_h_h
	{65}	long	@i_mov_h_l
	{66}	long	@i_mov_h_m
	{67}	long	@i_mov_h_a
	{68}	long	@i_mov_l_b
	{69}	long	@i_mov_l_c
	{6a}	long	@i_mov_l_d
	{6b}	long	@i_mov_l_e
	{6c}	long	@i_mov_l_h
	{6d}	long	@i_nop		' i_mov_l_l
	{6e}	long	@i_mov_l_m
	{6f}	long	@i_mov_l_a

	{70}	long	@i_mov_m_b
	{71}	long	@i_mov_m_c
	{72}	long	@i_mov_m_d
	{73}	long	@i_mov_m_e
	{74}	long	@i_mov_m_h
	{75}	long	@i_mov_m_l
	{76}	long	@i_halt		| (execute << PATCH_SHIFT)
	{77}	long	@i_mov_m_a
	{78}	long	@i_mov_a_b
	{79}	long	@i_mov_a_c
	{7a}	long	@i_mov_a_d
	{7b}	long	@i_mov_a_e
	{7c}	long	@i_mov_a_h
	{7d}	long	@i_mov_a_l
	{7e}	long	@i_mov_a_m
	{7f}	long	@i_nop		' i_mov_a_a

	{80}	long	@i_add_b
	{81}	long	@i_add_c
	{82}	long	@i_add_d
	{83}	long	@i_add_e
	{84}	long	@i_add_h
	{85}	long	@i_add_l
	{86}	long	@i_add_m
	{87}	long	@i_add_a
	{88}	long	@i_adc_b
	{89}	long	@i_adc_c
	{8a}	long	@i_adc_d
	{8b}	long	@i_adc_e
	{8c}	long	@i_adc_h
	{8d}	long	@i_adc_l
	{8e}	long	@i_adc_m
	{8f}	long	@i_adc_a

	{90}	long	@i_sub_b
	{91}	long	@i_sub_c
	{92}	long	@i_sub_d
	{93}	long	@i_sub_e
	{94}	long	@i_sub_h
	{95}	long	@i_sub_l
	{96}	long	@i_sub_m
	{97}	long	@i_sub_a
	{98}	long	@i_sbb_b
	{99}	long	@i_sbb_c
	{9a}	long	@i_sbb_d
	{9b}	long	@i_sbb_e
	{9c}	long	@i_sbb_h
	{9d}	long	@i_sbb_l
	{9e}	long	@i_sbb_m
	{9f}	long	@i_sbb_a

	{a0}	long	@i_ana_b
	{a1}	long	@i_ana_c
	{a2}	long	@i_ana_d
	{a3}	long	@i_ana_e
	{a4}	long	@i_ana_h
	{a5}	long	@i_ana_l
	{a6}	long	@i_ana_m
	{a7}	long	@i_ana_a
	{a8}	long	@i_xra_b
	{a9}	long	@i_xra_c
	{aa}	long	@i_xra_d
	{ab}	long	@i_xra_e
	{ac}	long	@i_xra_h
	{ad}	long	@i_xra_l
	{ae}	long	@i_xra_m
	{af}	long	@i_xra_a

	{b0}	long	@i_ora_b
	{b1}	long	@i_ora_c
	{b2}	long	@i_ora_d
	{b3}	long	@i_ora_e
	{b4}	long	@i_ora_h
	{b5}	long	@i_ora_l
	{b6}	long	@i_ora_m
	{b7}	long	@i_ora_a
	{b8}	long	@i_cmp_b
	{b9}	long	@i_cmp_c
	{ba}	long	@i_cmp_d
	{bb}	long	@i_cmp_e
	{bc}	long	@i_cmp_h
	{bd}	long	@i_cmp_l
	{be}	long	@i_cmp_m
	{bf}	long	@i_cmp_a

	{c0}	long	@i_rnz
	{c1}	long	@i_pop_b
	{c2}	long	@i_jnz		| (patch_jp_imm16 << PATCH_SHIFT)
	{c3}	long	@i_jmp		| (patch_jp_imm16 << PATCH_SHIFT)
	{c4}	long	@i_cnz		| (patch_imm16 << PATCH_SHIFT)
	{c5}	long	@i_push_b
	{c6}	long	@i_adi_imm8	| (patch_imm8 << PATCH_SHIFT)
	{c7}	long	@i_rst_0	| (execute << PATCH_SHIFT)
	{c8}	long	@i_rz
	{c9}	long	@i_ret		| (execute << PATCH_SHIFT)
	{ca}	long	@i_jz		| (patch_jp_imm16 << PATCH_SHIFT)
#ifdef	CPU_8085
	{cb}	long	@i_rst_v
#else
	{c3}	long	@i_jmp		| (patch_jp_imm16 << PATCH_SHIFT)
#endif
	{cc}	long	@i_cz		| (patch_imm16 << PATCH_SHIFT)
	{cd}	long	@i_call		| (execute_imm16 << PATCH_SHIFT)
	{ce}	long	@i_aci_imm8	| (patch_imm8 << PATCH_SHIFT)
	{cf}	long	@i_rst_1	| (execute << PATCH_SHIFT)

	{d0}	long	@i_rnc
	{d1}	long	@i_pop_d
	{d2}	long	@i_jnc		| (patch_jp_imm16 << PATCH_SHIFT)
	{d3}	long	@i_out_port_a	| (patch_imm8 << PATCH_SHIFT)
	{d4}	long	@i_cnc		| (patch_imm16 << PATCH_SHIFT)
	{d5}	long	@i_push_d
	{d6}	long	@i_sui_imm8	| (patch_imm8 << PATCH_SHIFT)
	{d7}	long	@i_rst_2	| (execute << PATCH_SHIFT)
	{d8}	long	@i_rc
#ifdef	CPU_8085
	{d9}	long	@i_shlx
#else
	{d9}	long	@i_ret		| (execute << PATCH_SHIFT)
#endif
	{da}	long	@i_jc		| (patch_jp_imm16 << PATCH_SHIFT)
	{db}	long	@i_in_a_port	| (patch_imm8 << PATCH_SHIFT)
	{dc}	long	@i_cc		| (patch_imm16 << PATCH_SHIFT)
#ifdef	CPU_8085
	{dd}	long	@i_jnx		| (patch_imm16 << PATCH_SHIFT)
#else
	{cd}	long	@i_call		| (execute_imm16 << PATCH_SHIFT)
#endif
	{de}	long	@i_sbi_imm8	| (patch_imm8 << PATCH_SHIFT)
	{df}	long	@i_rst_3	| (execute << PATCH_SHIFT)

	{e0}	long	@i_rpo
	{e1}	long	@i_pop_h
	{e2}	long	@i_jpo		| (patch_jp_imm16 << PATCH_SHIFT)
	{e3}	long	@i_xthl
	{e4}	long	@i_cpo		| (patch_imm16 << PATCH_SHIFT)
	{e5}	long	@i_push_h
	{e6}	long	@i_ani_imm8	| (patch_imm8 << PATCH_SHIFT)
	{e7}	long	@i_rst_4	| (execute << PATCH_SHIFT)
	{e8}	long	@i_rpe
	{e9}	long	@i_pchl		| (execute << PATCH_SHIFT)
	{ea}	long	@i_jpe		| (patch_jp_imm16 << PATCH_SHIFT)
	{eb}	long	@i_xchg
	{ec}	long	@i_cpe		| (patch_imm16 << PATCH_SHIFT)
#ifdef	CPU_8085
	{ed}	long	@i_lhlx
#else
	{cd}	long	@i_call		| (execute_imm16 << PATCH_SHIFT)
#endif
	{ee}	long	@i_xri_imm8	| (patch_imm8 << PATCH_SHIFT)
	{ef}	long	@i_rst_5	| (execute << PATCH_SHIFT)

	{f0}	long	@i_rp
	{f1}	long	@i_pop_a
	{f2}	long	@i_jp		| (patch_jp_imm16 << PATCH_SHIFT)
	{f3}	long	@i_di
	{f4}	long	@i_cp		| (patch_imm16 << PATCH_SHIFT)
	{f5}	long	@i_push_a
	{f6}	long	@i_ori_imm8	| (patch_imm8 << PATCH_SHIFT)
	{f7}	long	@i_rst_6	| (execute << PATCH_SHIFT)
	{f8}	long	@i_rm
	{f9}	long	@i_sphl
	{fa}	long	@i_jm		| (patch_jp_imm16 << PATCH_SHIFT)
	{fb}	long	@i_ei
	{fc}	long	@i_cm		| (patch_imm16 << PATCH_SHIFT)
#ifdef	CPU_8085
	{fd}	long	@i_jx		| (patch_imm16 << PATCH_SHIFT)
#else
	{cd}	long	@i_call		| (execute_imm16 << PATCH_SHIFT)
#endif
	{fe}	long	@i_cmi_imm8	| (patch_imm8 << PATCH_SHIFT)
	{ff}	long	@i_rst_7	| (execute << PATCH_SHIFT)
